#pragma once

ivec2	normalizedToPixelCoord(vec2 uv)
{
	return ivec2(uv * GET_CONSTANT(BlurInfo, ContextSize));
}

vec4 	blur5MS(SAMPLER2DMS_DCL_ARG(image), vec2 uv, vec2 resolution, vec2 direction)
{
	vec4 color = VEC4_ZERO;
	vec2 off1 = 1.3333333333333333 * direction;
	for (int i = 0; i < SAMPLE_COUNT; ++i)
	{
		color += SAMPLEMS(image, normalizedToPixelCoord(uv), i) * 0.29411764705882354;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv + (off1 / resolution)), i) * 0.35294117647058826;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv - (off1 / resolution)), i) * 0.35294117647058826;
	}
	return color / float(SAMPLE_COUNT);
}

vec4 	blur9MS(SAMPLER2DMS_DCL_ARG(image), vec2 uv, vec2 resolution, vec2 direction)
{
	vec4 color = VEC4_ZERO;
	vec2 off1 = 1.3846153846 * direction;
	vec2 off2 = 3.2307692308 * direction;
	for (int i = 0; i < SAMPLE_COUNT; ++i)
	{
		color += SAMPLEMS(image, normalizedToPixelCoord(uv), i) * 0.2270270270;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv + (off1 / resolution)), i) * 0.3162162162;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv - (off1 / resolution)), i) * 0.3162162162;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv + (off2 / resolution)), i) * 0.0702702703;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv - (off2 / resolution)), i) * 0.0702702703;
	}
	return color / float(SAMPLE_COUNT);
}

vec4 	blur13MS(SAMPLER2DMS_DCL_ARG(image), vec2 uv, vec2 resolution, vec2 direction)
{
	vec4 color = VEC4_ZERO;
	vec2 off1 = 1.411764705882353 * direction;
	vec2 off2 = 3.2941176470588234 * direction;
	vec2 off3 = 5.176470588235294 * direction;
	for (int i = 0; i < SAMPLE_COUNT; ++i)
	{
		color += SAMPLEMS(image, normalizedToPixelCoord(uv), i) * 0.1964825501511404;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv + (off1 / resolution)), i) * 0.2969069646728344;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv - (off1 / resolution)), i) * 0.2969069646728344;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv + (off2 / resolution)), i) * 0.09447039785044732;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv - (off2 / resolution)), i) * 0.09447039785044732;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv + (off3 / resolution)), i) * 0.010381362401148057;
		color += SAMPLEMS(image, normalizedToPixelCoord(uv - (off3 / resolution)), i) * 0.010381362401148057;
	}
	return color / float(SAMPLE_COUNT);
}

vec4 	blur5(SAMPLER2D_DCL_ARG(image), vec2 uv, vec2 resolution, vec2 direction)
{
	vec4 color = VEC4_ZERO;
	vec2 off1 = 1.3333333333333333 * direction;
	color += SAMPLE(image, uv) * 0.29411764705882354;
	color += SAMPLE(image, uv + (off1 / resolution)) * 0.35294117647058826;
	color += SAMPLE(image, uv - (off1 / resolution)) * 0.35294117647058826;
	return color; 
}

vec4 	blur9(SAMPLER2D_DCL_ARG(image), vec2 uv, vec2 resolution, vec2 direction)
{
	vec4 color = VEC4_ZERO;
	vec2 off1 = 1.3846153846 * direction;
	vec2 off2 = 3.2307692308 * direction;
	color += SAMPLE(image, uv) * 0.2270270270;
	color += SAMPLE(image, uv + (off1 / resolution)) * 0.3162162162;
	color += SAMPLE(image, uv - (off1 / resolution)) * 0.3162162162;
	color += SAMPLE(image, uv + (off2 / resolution)) * 0.0702702703;
	color += SAMPLE(image, uv - (off2 / resolution)) * 0.0702702703;
	return color;
}

vec4	blur13(SAMPLER2D_DCL_ARG(image), vec2 uv, vec2 resolution, vec2 direction)
{
	vec4 color = VEC4_ZERO;
	vec2 off1 = 1.411764705882353 * direction;
	vec2 off2 = 3.2941176470588234 * direction;
	vec2 off3 = 5.176470588235294 * direction;
	color += SAMPLE(image, uv) * 0.1964825501511404;
	color += SAMPLE(image, uv + (off1 / resolution)) * 0.2969069646728344;
	color += SAMPLE(image, uv - (off1 / resolution)) * 0.2969069646728344;
	color += SAMPLE(image, uv + (off2 / resolution)) * 0.09447039785044732;
	color += SAMPLE(image, uv - (off2 / resolution)) * 0.09447039785044732;
	color += SAMPLE(image, uv + (off3 / resolution)) * 0.010381362401148057;
	color += SAMPLE(image, uv - (off3 / resolution)) * 0.010381362401148057;
	return color;
}
