#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) smooth in vec3 _fin_fragNormal;
layout(location = 1) smooth in vec2 _fin_fragTexCoord;
layout(location = 2) smooth in vec4 _fin_fragWorldPosition;

struct SFragmentInput
{
	vec3 fragNormal;
	vec2 fragTexCoord;
	vec4 fragWorldPosition;
};

layout(location = 0) out vec4 _fout_outDiffuse;
layout(location = 1) out float _fout_outDepth;
layout(location = 2) out vec4 _fout_outEmissive;
layout(location = 3) out vec4 _fout_outNormalSpec;

struct SFragmentOutput
{
	vec4 outDiffuse;
	float outDepth;
	vec4 outEmissive;
	vec4 outNormalSpec;
};

layout( std140) uniform UMeshFragmentConstant
{
	float Roughness;
	float Metalness;
} MeshFragmentConstant;




vec2 PackNormalSpheremap(vec3 normal)
{
	const float eps = 0.001f;
	const float sqrt3e = eps * sqrt(3.0f);

	normal = (normal + sign(normal) * vec3(eps, eps, eps)) / (1.0 + sqrt3e);
	return normalize(normal.xy) * sqrt(-(normal.z) * 0.5 + 0.5) * 0.5 + 0.5;
}

void FragmentMain(in SFragmentInput fInput, out SFragmentOutput fOutput)
{


	vec4 diffuseColor;




	diffuseColor = vec4(1.0, 1.0, 1.0, 1.0);




	float roughness =  MeshFragmentConstant.Roughness ;
	float metalness =  MeshFragmentConstant.Metalness ;
	fOutput.outDiffuse = diffuseColor;
	fOutput.outNormalSpec = vec4(PackNormalSpheremap(normalize(fInput.fragNormal)), roughness, metalness);
	fOutput.outDepth = fInput.fragWorldPosition.z / fInput.fragWorldPosition.w;
	fOutput.outEmissive = vec4(0,0,0,0);
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragNormal = _fin_fragNormal;
	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.fragWorldPosition = _fin_fragWorldPosition;
	FragmentMain(fInput, fOutput);
	_fout_outDiffuse = fOutput.outDiffuse;
	_fout_outDepth = fOutput.outDepth;
	_fout_outEmissive = fOutput.outEmissive;
	_fout_outNormalSpec = fOutput.outNormalSpec;
}
