#version 330
#extension GL_ARB_separate_shader_objects : enable
precision highp float;
precision highp sampler2D;
precision highp sampler2DMS;
precision highp samplerCube;

layout(location = 0) in vec3 _vin_Position;
layout(location = 1) in vec3 _vin_Color;

struct SVertexInput
{
	vec3 Position;
	vec3 Color;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec3 _vout_fragColor;

struct SVertexOutput
{
	vec3 fragColor;
	vec4 VertexPosition;
};


layout( std140) uniform UGizmoConstant
{
	mat4 ModelViewProj;
	vec4 HoveredColor;
	vec4 GrabbedColor;
} GizmoConstant;

void VertexMain(in SVertexInput vInput, out SVertexOutput vOutput)
{
	vec4 grabbedColor =  GizmoConstant.GrabbedColor ;
	vec4 hoveredColor =  GizmoConstant.HoveredColor ;

	vOutput.VertexPosition =  (( GizmoConstant.ModelViewProj ) * (vec4(vInput.Position, 1.))) ;
	if (hoveredColor.a == 1.)
		vOutput.fragColor = hoveredColor.rgb;
	else if (grabbedColor.a == 1.)
		vOutput.fragColor = grabbedColor.rgb;
	else
		vOutput.fragColor = vInput.Color;
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.Position = _vin_Position;
	vInput.Color = _vin_Color;
	vInput.VertexIndex = gl_VertexID;
	vInput.InstanceId = uint(gl_InstanceID);
	vOutput.fragColor = vec3(0, 0, 0);
	VertexMain(vInput, vOutput);
	_vout_fragColor = vOutput.fragColor;
	gl_Position = vOutput.VertexPosition;
}
