#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) in vec3 _vin_Position;
layout(location = 1) in vec3 _vin_LightPosition;
layout(location = 2) in float _vin_LightRadius;
layout(location = 3) in vec3 _vin_LightColor;

struct SVertexInput
{
	vec3 Position;
	vec3 LightPosition;
	float LightRadius;
	vec3 LightColor;
	int VertexIndex;
	uint InstanceId;
};

layout(location = 0) smooth out vec4 _vout_fragWorldPosition;
layout(location = 1) flat out vec3 _vout_fragLightPosition;
layout(location = 2) flat out float _vout_fragLightRadius;
layout(location = 3) flat out vec3 _vout_fragLightColor;

struct SVertexOutput
{
	vec4 fragWorldPosition;
	vec3 fragLightPosition;
	float fragLightRadius;
	vec3 fragLightColor;
	vec4 VertexPosition;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 UserToLHZ;
	mat4 LHZToUser;
	mat4 BillboardingView;
	mat4 PackNormalView;
	mat4 UnpackNormalView;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	float Handedness;
} SceneInfo;



void VertexMain( in SVertexInput  vInput,  out SVertexOutput  vOutput  )
{

	vec3 transformedPos = vInput.Position * 1.1 * vInput.LightRadius + vInput.LightPosition;
	mat4 viewProj =  SceneInfo.ViewProj ;
	vOutput.VertexPosition =  ((viewProj) * (vec4(transformedPos, 1.0))) ;
	vOutput.fragWorldPosition = vOutput.VertexPosition;
	vOutput.fragLightPosition = vInput.LightPosition;
	vOutput.fragLightRadius = vInput.LightRadius;
	vOutput.fragLightColor = vInput.LightColor;
}


void main()
{
	SVertexInput vInput;
	SVertexOutput vOutput;

	vInput.Position = _vin_Position;
	vInput.LightPosition = _vin_LightPosition;
	vInput.LightRadius = _vin_LightRadius;
	vInput.LightColor = _vin_LightColor;
	vInput.VertexIndex = gl_VertexID;
	vInput.InstanceId = uint(gl_InstanceID);
	vOutput.fragWorldPosition = vec4(0, 0, 0, 0);
	vOutput.fragLightPosition = vec3(0, 0, 0);
	vOutput.fragLightRadius = float(0);
	vOutput.fragLightColor = vec3(0, 0, 0);
	VertexMain(vInput, vOutput);
	_vout_fragWorldPosition = vOutput.fragWorldPosition;
	_vout_fragLightPosition = vOutput.fragLightPosition;
	_vout_fragLightRadius = vOutput.fragLightRadius;
	_vout_fragLightColor = vOutput.fragLightColor;
	gl_Position = vOutput.VertexPosition;
}
